#ifndef XCACHE_IO_H__
#define XCACHE_IO_H__

#include "ocf_env.h"
#include "ocf/ocf.h"

enum entry_type {
	XCACHE_IO_ENTRY,
	XCACHE_BACKDEV_IO_ENTRY,
	OCF_REQ_ENTRY,
};

struct queue_entry {
	enum entry_type type;
	struct list_head list;
};

#define INLINE_FLUSH_LINES 4
struct xcache_io;
typedef void (*xcache_io_end_fn)(struct xcache_io *io, int error);
typedef int (*xcache_io_if)(struct xcache_io *io);
struct xcache_io {
	// queue_entry
	enum entry_type type;
	struct list_head queue_list;

	xcache_io_end_fn end;
	xcache_io_if io_if;
	env_atomic remaining;
	ctx_data_t *data;
	int error;

	ocf_queue_t io_queue;
	ocf_cache_t cache;
	ocf_core_t core;
	uint64_t start_addr;
	uint64_t size;
	uint8_t rw;
	uint8_t flags;

	ocf_cache_line_t flush_lines[INLINE_FLUSH_LINES];
	uint64_t flush_line_num;
};

struct backdev_io_end_arg {
	uint64_t addr;
	uint64_t size;
	int error;
};
struct xcache_backdev_io;
typedef int (*backdev_io_end_fn)(struct xcache_backdev_io *io, struct backdev_io_end_arg *arg);
typedef void (*backdev_io_res_fn)(struct xcache_backdev_io *io);
enum xcache_dir {
	XCACHE_RD = 0,
	XCACHE_WR,
	XCACHE_FLUSH,
};
struct xcache_backdev_io {
	// queue_entry
	enum entry_type type;
	struct list_head free_list;

	struct xcache_io *xcache_io;
	backdev_io_end_fn end;

	ocf_cache_line_t line;
	ctx_data_t *data;

	union {
		/* for xcache lock */
		struct {
			backdev_io_res_fn io_res;
			uint64_t addr;
			uint64_t size;
		};
		/* for io_end callback */
		struct {
			int dev;
			enum xcache_dir dir;
		};
	};

	void *priv;
};

void xcache_backdev_io_end(struct xcache_backdev_io *bd_io, struct backdev_io_end_arg *arg);
void xcache_submit_io(struct xcache_io *io);
void spdk_backdev_submit_io(struct xcache_backdev_io *io_base, bool to_cache, uint64_t addr, uint64_t len, uint64_t offset, uint8_t dir);

#endif
