#ifndef EVICTING_OPS_H_
#define EVICTING_OPS_H_

#include <stddef.h>
#include "../xcache.h"

struct evicting_policy_ops {
	int (*init)(xcache_context_t *ctx);
	void (*line_accessed)(xcache_context_t *ctx, xcache_line_t line);
	void (*line_dirty)(xcache_context_t *ctx, xcache_line_t line);
	void (*line_clean)(xcache_context_t *ctx, xcache_line_t line);
	int (*line_to_clean)(xcache_context_t *ctx, xcache_line_t *lines, int line_num);
	int (*line_to_evict)(xcache_context_t *ctx, xcache_line_t *lines, int line_num);
	void (*fini)(xcache_context_t *ctx);
};

extern struct evicting_policy_ops *g_evicting_policy;

static inline void evicting_set(struct evicting_policy_ops *policy)
{
	g_evicting_policy = policy;
}

static inline void evicting_init(xcache_context_t *ctx)
{
	g_evicting_policy->init(ctx);
}

static inline void evicting_line_accessed(xcache_context_t *ctx, xcache_line_t line)
{
	g_evicting_policy->line_accessed(ctx, line);
}

static inline void evicting_line_dirty(xcache_context_t *ctx, xcache_line_t line)
{
	g_evicting_policy->line_dirty(ctx, line);
}

static inline void evicting_line_clean(xcache_context_t *ctx, xcache_line_t line)
{
	g_evicting_policy->line_clean(ctx, line);
}

static inline xcache_line_t evicting_line_to_clean(xcache_context_t *ctx,
		xcache_line_t *lines, int line_num)
{
	return g_evicting_policy->line_to_clean(ctx, lines, line_num);
}

static inline xcache_line_t evicting_line_to_evict(xcache_context_t *ctx,
		xcache_line_t *lines, int line_num)
{
	return g_evicting_policy->line_to_evict(ctx, lines, line_num);
}

static inline void evicting_fini(xcache_context_t *ctx)
{
	return g_evicting_policy->fini(ctx);
}

#endif
