#include "ocf/ocf.h"

#include "xcache_metadata.h"
#include "metadata.h"
#include "metadata_internal.h"
#include "../engine/xcache_engine_common.h" /* xcache_io_xxx */
#include "../ocf_cache_priv.h"

void xcache_metadata_flush_asynch(struct ocf_cache *cache,
                struct ocf_metadata_io_context *io_ctx,
	       struct ocf_metadata_line_getter *line_getter)
{
        int result = 0;
        struct ocf_metadata_ctrl *ctrl = NULL;

        ctrl = (struct ocf_metadata_ctrl *) cache->metadata.priv;
        result |= ocf_metadata_raw_flush_do_asynch_common(cache,
                        &(ctrl->raw_desc[metadata_segment_collision]),
			io_ctx, line_getter);
        if (result) {
                ocf_metadata_error(cache);
                ocf_cache_log(cache, log_err, "Metadata Flush ERROR\n");
        }
}

static inline bool is_power_of_two(uint64_t n)
{
	return (((n - 1) & n) == 0);
}

int xcache_io_add_flush_line(struct xcache_io *io, ocf_cache_line_t line)
{
	ocf_cache_line_t *flush_lines;
	int i;

	if (io->flush_line_num < INLINE_FLUSH_LINES) {
		flush_lines = io->flush_lines;
	} else if (is_power_of_two(io->flush_line_num)) {
		flush_lines = env_malloc(sizeof(ocf_cache_line_t) * (io->flush_line_num << 1), 0);
		if (flush_lines == NULL) {
			return -ENOMEM;
		}
		for (i = 0; i < io->flush_line_num; i++) {
			flush_lines[i] = io->flush_lines[i];
		}
		*(ocf_cache_line_t **)io->flush_lines = flush_lines;
	} else {
		flush_lines = *(ocf_cache_line_t **)io->flush_lines;
	}

	flush_lines[io->flush_line_num++] = line;
	return 0;
}

static int io_line_num(void *getter)
{
	struct xcache_io *io = (struct xcache_io *)getter;

	return io->flush_line_num;
}

static bool io_flush_line(void *getter, int index, ocf_cache_line_t *line)
{
	struct xcache_io *io = (struct xcache_io *)getter;
	ocf_cache_line_t *flush_lines;

	flush_lines = xcache_io_get_flush_line(io);
	*line = io->flush_lines[index];
	return true;
}

void xcache_io_metadata_flush(struct xcache_io *io, ocf_metadata_io_ctx_end_t io_end)
{
	struct ocf_metadata_io_context io_ctx = {
		.io = (void *)io,
		.io_flags = xcache_io_flags(io),
		.io_end = io_end,
		.queue = xcache_io_queue(io),
	};
	struct ocf_metadata_line_getter line_getter = {
		.getter = (void *)io,
		.get_line_num = io_line_num,
		.get_flush_line = io_flush_line,
	};

	xcache_io_get(io);
	xcache_metadata_flush_asynch(xcache_io_cache(io), &io_ctx, &line_getter);
}
