#ifndef XCACHE_H_
#define XCACHE_H_

#include <stdint.h>
#include "ocf/ocf_types.h"
#include "./ocf_cache_priv.h"

#include "ocf/xcache.h"
#include "xcache_cleaner.h"
#include "qos/qos.h"

typedef ocf_cache_line_t xcache_line_t;

typedef struct xcache_context {
	uint8_t line_size_shift;
	void *xcache_evicting;
	ocf_cache_t cache;
	struct xcache_cleaning_ctx cleaning_ctx;
	struct xcache_qos qos;
} xcache_context_t;

static inline xcache_context_t *xcache_get_ctx(ocf_cache_t cache)
{
	return cache->xcache_ctx;
}

static inline void *xcache_get_evicting(xcache_context_t *ctx)
{
	return ctx->xcache_evicting;
}

static inline void xcache_set_evicting(xcache_context_t *ctx, void *evicting)
{
	ctx->xcache_evicting = evicting;
}

static inline uint64_t xcache_addr_to_line(ocf_cache_t cache, uint64_t addr)
{
	xcache_context_t *ctx = cache->xcache_ctx;
	return addr >> ctx->line_size_shift;
}

static inline uint64_t xcache_line_to_addr(ocf_cache_t cache, uint64_t line)
{
	xcache_context_t *ctx = cache->xcache_ctx;
	return line << ctx->line_size_shift;
}

static inline struct xcache_cleaning_ctx *xcache_get_cleaning_ctx(ocf_cache_t cache)
{
	xcache_context_t *xcache_ctx = cache->xcache_ctx;
	return &xcache_ctx->cleaning_ctx;
}

int xcache_init(ocf_cache_t cache);
void xcache_fini(ocf_cache_t cache);

#endif
