#ifndef XCACHE_CLEANER_H_
#define XCACHE_CLEANER_H_

#include "ocf/ocf_io.h"
#include "ocf/ocf_types.h"
#include "utils/utils_refcnt.h"

#include "ocf/xcache.h"

#define XCACHE_CLEAN_SIZE 32

struct clean_sort_data {
	ocf_core_id_t core_id;
	uint64_t core_line;
	ocf_cache_line_t line;
};

typedef void (*cleaning_end_t)(void *arg);
struct xcache_cleaning_ctx {
	ocf_cache_t cache;
	struct ocf_refcnt counter;
	struct xcache_io io;
	struct xcache_backdev_io base_io;
	struct clean_sort_data data[XCACHE_CLEAN_SIZE];
	int count;
	cleaning_end_t end;
	void *arg;
};

int xcache_cleaner_clean(ocf_cache_t cache, ocf_queue_t q,
		struct xcache_cleaning_ctx *cleaning_ctx);

static inline int xcache_cleaner_prepare(struct xcache_cleaning_ctx *cleaning_ctx)
{
	if (ocf_refcnt_inc(&cleaning_ctx->counter) == 1) {
		return 0;
	}

	ocf_refcnt_dec(&cleaning_ctx->counter);
	return -1;
}

static inline void xcache_cleaner_complete(struct xcache_cleaning_ctx *cleaning_ctx)
{
	ocf_refcnt_dec(&cleaning_ctx->counter);
}

static inline int xcache_cleaner_fill(struct xcache_cleaning_ctx *cleaning_ctx,
		ocf_cache_line_t *clines, int line_num)
{
	struct clean_sort_data *data = cleaning_ctx->data;
	int num = line_num < XCACHE_CLEAN_SIZE ? line_num : XCACHE_CLEAN_SIZE;
	int i;

	for (i = 0; i < num; i++) {
		data[i].line = clines[i];
	}
	cleaning_ctx->count = num;
	return num;
}

static inline void xcache_cleaner_set_end(struct xcache_cleaning_ctx *cleaning_ctx,
		cleaning_end_t end, void *arg)
{
	cleaning_ctx->end = end;
	cleaning_ctx->arg = arg;
}

#endif /* XCACHE_CLEANER_H_ */
