#include "ocf/ocf.h"
#include "ocf_cache_priv.h"
#include "metadata/metadata.h"
#include "engine/xcache_engine.h"
#include "engine/xcache_engine_common.h"

#include "ocf/xcache.h"
#include "xcache_queue.h"
#include "qos/qos.h"

void xcache_submit_io(struct xcache_io *io)
{
	ocf_core_t core = xcache_io_core(io);
	ocf_cache_t cache = xcache_io_cache(io);
	int ret;

	OCF_CHECK_NULL(io);

	if (unlikely(!env_bit_test(ocf_cache_state_running,
					&cache->cache_state))) {
		xcache_io_end(io, -OCF_ERR_CACHE_NOT_AVAIL);
		return;
	}

	xcache_get_io_if(io, cache->conf_meta->cache_mode);
	if (io->io_if == NULL) {
		xcache_io_end(io, -OCF_ERR_INVAL);
		return;
	}

	env_atomic_set(&io->remaining, 1);
	xcache_queue_push_xcache_io_back(io, true);
}

void xcache_backdev_io_end(struct xcache_backdev_io *bd_io, struct backdev_io_end_arg *arg)
{
	struct xcache_io *io = bd_io->xcache_io;
	xcache_context_t *xcache_ctx = backdev_io_to_xcache_ctx(bd_io);

	xcache_qos_load_sub(&xcache_ctx->qos, bd_io->dev, bd_io->dir, arg->size);

	io->error |= arg->error;
	bd_io->end(bd_io, arg);
	xcache_io_put(io);
}
